@@ -1,6 +1,6 @@
 
 /*
- *  $Id: expPrint.c,v 1.37 2003/12/07 19:56:06 bkorb Exp $
+ *  $Id: expPrint.c,v 1.42 2004/08/08 15:10:50 bkorb Exp $
  *
  *  The following code is necessary because the user can give us
  *  a printf format requiring a string pointer yet fail to provide
@@ -10,7 +10,7 @@
  */
 
 /*
- *  AutoGen copyright 1992-2003 Bruce Korb
+ *  AutoGen copyright 1992-2004 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -75,7 +75,7 @@ safePrintf( char** ppzBuf, char* pzFmt, void** argV )
              *  THEN perform that fprintf
              */
             if (sigsetjmp( printJumpEnv, 0 ) == 0)
-                fprintf(pfTrace, zBadFmt, pzProg, strsignal( faultType ), pzFmt);
+                fprintf(pfTrace, zBadFmt, pzProg, strsignal(faultType), pzFmt);
 
             /*
              *  The "sprintfv" command below faulted, so we exit
@@ -85,7 +85,8 @@ safePrintf( char** ppzBuf, char* pzFmt, void** argV )
     }
 
     {
-        size_t printSize = asprintfv( ppzBuf, pzFmt, (void*)argV );
+        size_t printSize = asprintfv( ppzBuf, pzFmt,
+                                      (snv_constpointer*)argV );
         if ((printSize & ~0xFFFFFF) != 0)
             AG_ABEND( aprf( "asprintfv returned 0x%08X\n", printSize ));
 
@@ -116,7 +117,7 @@ run_printf( char* pzFmt, int len, SCM alist )
         switch (gh_type_e( car )) {
         default:
         case GH_TYPE_UNDEFINED:
-            *(argp++) = (void*)"???";
+            *(argp++) = "???";
             break;
 
         case GH_TYPE_BOOLEAN:
@@ -124,29 +125,29 @@ run_printf( char* pzFmt, int len, SCM alist )
             break;
 
         case GH_TYPE_CHAR:
-            *(argp++) = (void*)(uintptr_t)gh_scm2char( car );
+            *(char*)(argp++) = gh_scm2char( car );
             break;
 
         case GH_TYPE_PAIR:
-            *(argp++) = (void*)"..";
+            *(argp++) = "..";
             break;
 
         case GH_TYPE_NUMBER:
-            *(argp++) = (void*)gh_scm2ulong( car );
+            *(unsigned long*)(argp++) = gh_scm2ulong( car );
             break;
 
         case GH_TYPE_SYMBOL:
         case GH_TYPE_STRING:
-            *(argp++) = (void*)ag_scm2zchars( car, "printf str" );
+            *(argp++) = ag_scm2zchars( car, "printf str" );
             break;
 
         case GH_TYPE_PROCEDURE:
-            *(argp++) = (void*)"(*)()";
+            *(argp++) = "(*)()";
             break;
 
         case GH_TYPE_VECTOR:
         case GH_TYPE_LIST:
-            *(argp++) = (void*)"...";
+            *(argp++) = "...";
             break;
         }
     }
