@@ -1,6 +1,6 @@
 
 /*
- *  $Id: autoopts.c,v 2.65 2004/01/01 20:06:00 bkorb Exp $
+ *  $Id: autoopts.c,v 2.74 2004/10/15 01:54:16 bkorb Exp $
  *
  *  This file contains all of the routines that must be linked into
  *  an executable to use the generated option processing.  The optional
@@ -9,7 +9,7 @@
  */
 
 /*
- *  Automated Options copyright 1992-2003 Bruce Korb
+ *  Automated Options copyright 1992-2004 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -69,12 +69,6 @@ findOptDesc( tOptions* pOpts, tOptState* pOptState );
 STATIC tSuccess
 nextOption( tOptions* pOpts, tOptState* pOptState );
 
-STATIC tSuccess
-doImmediateOpts( tOptions* pOpts );
-
-STATIC void
-doEnvPresets( tOptions* pOpts, teEnvPresetType type );
-
 STATIC void
 doRcFiles( tOptions* pOpts );
 
@@ -104,6 +98,17 @@ handleOption( tOptions* pOpts, tOptState* pOptState )
 
     pOD->pzLastArg =  pOptState->pzOptArg;
 
+    /*
+     *  IF we are presetting options, then we will ignore any un-presettable
+     *  options.  They are the ones either marked as such, or equivalenced to
+     *  a different option.
+     */
+    if (  ((pOpts->fOptSet & OPTPROC_PRESETTING) != 0)
+       && (  ((pOD->fOptState & OPTST_NO_INIT) != 0)
+          || (pOD->optEquivIndex != NO_EQUIVALENT)
+       )  )
+        return PROBLEM;
+
     /*
      *  IF this is an equivalence class option,
      *  THEN
@@ -113,15 +118,34 @@ handleOption( tOptions* pOpts, tOptState* pOptState )
      *      set the pointer to the equivalence class base
      */
     if (pOD->optEquivIndex != NO_EQUIVALENT) {
-        tOptDesc*  p = pOpts->pOptDesc + pOD->optEquivIndex;
+        tOptDesc* p = pOpts->pOptDesc + pOD->optEquivIndex;
+
+        if (p->optActualIndex == NO_EQUIVALENT) {
+            /*
+             *  First time through, copy over the state
+             *  and add in the equivalence flag
+             */
+            p->optActualValue = pOD->optValue;
+            p->optActualIndex = pOD->optIndex;
+            p->optCookie      = pOD->optCookie;
+            pOptState->flags |= OPTST_EQUIVALENCE;          
+        }
+
+        else if (p->optActualIndex != pOD->optIndex) {
+            /*
+             *  IF this option has been previously equivalenced and
+             *     it was not the same equivalenced-to option,
+             *  THEN we have a usage problem.
+             */
+            fprintf( stderr, (char*)zMultiEquiv, p->pz_Name, pOD->pz_Name,
+                     (pOpts->pOptDesc + p->optActualIndex)->pz_Name);
+            return FAILURE;
+        }
 
         /*
-         *  Add in the equivalence flag
+         *  Copy the most recent option argument
          */
-        pOptState->flags |= OPTST_EQUIVALENCE;
-        p->pzLastArg      = pOD->pzLastArg;
-        p->optActualValue = pOD->optValue;
-        p->optActualIndex = pOD->optIndex;
+        p->pzLastArg = pOD->pzLastArg;
         pOD = p;
 
     } else {
@@ -138,8 +162,8 @@ handleOption( tOptions* pOpts, tOptState* pOptState )
      */
     if (  (pOD->fOptState & OPTST_DEFINED)
        && (++pOD->optOccCt > pOD->optMaxCt)  )  {
-        const char* pzEqv = (pOD->optEquivIndex != NO_EQUIVALENT)
-                          ? zEquiv : zEquiv + sizeof( zEquiv )-1;
+        const char* pzEqv =
+            (pOD->optEquivIndex != NO_EQUIVALENT) ? zEquiv : zNil;
 
         if ((pOpts->fOptSet & OPTPROC_ERRSTOP) != 0) {
             const char* pzFmt = (pOD->optMaxCt > 1) ? zAtMost : zOnlyOne;
@@ -219,7 +243,7 @@ longOptionFind( tOptions* pOpts, char* pzOptName, tOptState* pOptState )
          *  THEN ...
          */
         else if (  (pOD->pz_DisableName != NULL)
-                && (strneqvcmp( pzOptName, pOD->pz_DisableName, nameLen ) == 0)
+                && (strneqvcmp(pzOptName, pOD->pz_DisableName, nameLen) == 0)
                 )  {
             disable  = AG_TRUE;
 
@@ -642,17 +666,12 @@ nextOption( tOptions* pOpts, tOptState* pOptState )
 /*
  *  doImmediateOpts - scan the command line for immediate action options
  */
-STATIC tSuccess
+LOCAL tSuccess
 doImmediateOpts( tOptions* pOpts )
 {
     pOpts->curOptIdx = 1;     /* start by skipping program name */
     pOpts->pzCurOpt  = NULL;
 
-    /*
-     *  when comparing long names, these are equivalent
-     */
-    strequate( zSepChars );
-
     /*
      *  Examine all the options from the start.  We process any options that
      *  are marked for immediate processing.
@@ -669,24 +688,9 @@ doImmediateOpts( tOptions* pOpts )
         /*
          *  IF this *is* an immediate-attribute option, then do it.
          */
-        switch (optState.flags & (OPTST_DISABLE_IMM|OPTST_IMM)) {
-        case 0:                   /* never */
+        if (! DO_IMMEDIATELY(optState.flags))
             continue;
 
-        case OPTST_DISABLE_IMM:   /* do enabled options later */
-            if ((optState.flags & OPTST_DISABLED) == 0)
-                continue;
-            break;
-
-        case OPTST_IMM:           /* do disabled options later */
-            if ((optState.flags & OPTST_DISABLED) != 0)
-                continue;
-            break;
-
-        case OPTST_DISABLE_IMM|OPTST_IMM: /* always */
-            break;
-        }
-
         if (! SUCCESSFUL( handleOption( pOpts, &optState )))
             break;
     } optionsDone:;
@@ -697,124 +701,39 @@ doImmediateOpts( tOptions* pOpts )
 }
 
 
-/*
- *  doEnvPresets - check for preset values from the envrionment
- *  This routine should process in all, immediate or normal modes....
- */
-STATIC void
-doEnvPresets( tOptions* pOpts, teEnvPresetType type )
+LOCAL tSuccess
+doRegularOpts( tOptions* pOpts )
 {
-    int        ct;
-    tOptState  st;
-    char*      pzFlagName;
-    size_t     spaceLeft;
-    char       zEnvName[ AO_NAME_SIZE ];
-
     /*
-     *  Finally, see if we are to look at the environment
-     *  variables for initial values.
+     *  Now, process all the options from our current position onward.
+     *  (This allows interspersed options and arguments for the few
+     *  non-standard programs that require it.)
      */
-    if ((pOpts->fOptSet & OPTPROC_ENVIRON) == 0)
-        return;
+    for (;;) {
+        tOptState optState = OPTSTATE_INITIALIZER;
 
-    ct  = pOpts->presetOptCt;
-    st.pOD = pOpts->pOptDesc;
-
-    pzFlagName = zEnvName
-        + snprintf( zEnvName, sizeof( zEnvName ), "%s_", pOpts->pzPROGNAME );
-    spaceLeft = AO_NAME_SIZE - (pzFlagName - zEnvName) - 1;
-
-    for (;ct-- > 0; st.pOD++) {
-        /*
-         *  If presetting is disallowed, then skip this entry
-         */
-        if ((st.pOD->fOptState & OPTST_NO_INIT) != 0)
-            continue;
-
-        /*
-         *  IF there is no such environment variable,
-         *  THEN skip this entry, too.
-         */
-        if (strlen( st.pOD->pz_NAME ) >= spaceLeft)
-            continue;
-
-        /*
-         *  Set up the option state
-         */
-        strcpy( pzFlagName, st.pOD->pz_NAME );
-        st.pzOptArg = getenv( zEnvName );
-        if (st.pzOptArg == NULL)
-            continue;
-        st.flags    = OPTST_PRESET | st.pOD->fOptState;
-        st.optType  = TOPT_UNDEFINED;
-        st.argType  = 0;
-
-        if (  (st.pOD->pz_DisablePfx != NULL)
-           && (streqvcmp( st.pzOptArg, st.pOD->pz_DisablePfx ) == 0)) {
-            st.flags |= OPTST_DISABLED;
-            st.pzOptArg = NULL;
-        }
-
-        switch (type) {
-        case ENV_IMM:
-            /*
-             *  Process only immediate actions
-             */
-            if (st.flags & OPTST_DISABLED) {
-                if ((st.flags & OPTST_DISABLE_IMM) == 0)
-                    continue;
-            } else {
-                if ((st.flags & OPTST_IMM) == 0)
-                    continue;
-            }
-            break;
-
-        case ENV_NON_IMM:
-            /*
-             *  Process only NON immediate actions
-             */
-            if (st.flags & OPTST_DISABLED) {
-                if ((st.flags & OPTST_DISABLE_IMM) != 0)
-                    continue;
-            } else {
-                if ((st.flags & OPTST_IMM) != 0)
-                    continue;
-            }
-            break;
-
-        default: /* process everything */
-            break;
+        switch (nextOption( pOpts, &optState )) {
+        case FAILURE: goto optionsDone;
+        case PROBLEM: return SUCCESS; /* no more args */
+        case SUCCESS: break;
         }
 
         /*
-         *  Make sure the option value string is persistent and consistent.
-         *  This may be a memory leak, but we cannot do anything about it.
-         *
-         *  The interpretation of the option value depends
-         *  on the type of value argument the option takes
+         *  IF this is not being processed normally (i.e. is immediate action)
+         *  THEN skip it (unless we are supposed to do it a second time).
          */
-        if (st.pzOptArg != NULL)
-            switch (st.pOD->optArgType) {
-            case ARG_MAY:
-                if (*st.pzOptArg == NUL) {
-                    st.pzOptArg = NULL;
-                    break;
-                }
-                /* FALLTHROUGH */
+        if (! DO_NORMALLY(optState.flags)) {
+            if (! DO_SECOND_TIME(optState.flags))
+                continue;
+            optState.pOD->optOccCt--; /* don't count last time */
+        }
 
-            case ARG_MUST:
-                if (*st.pzOptArg == NUL)
-                     st.pzOptArg = zNil;
-                else AGDUPSTR( st.pzOptArg, st.pzOptArg, "option argument" );
-                break;
-
-            default: /* no argument allowed */
-                st.pzOptArg = NULL;
-                break;
-            }
-
-        handleOption( pOpts, &st );
-    }
+        if (! SUCCESSFUL( handleOption( pOpts, &optState )))
+            break;
+    } optionsDone:;
+    if ((pOpts->fOptSet & OPTPROC_ERRSTOP) != 0)
+        (*pOpts->pUsageProc)( pOpts, EXIT_FAILURE );
+    return FAILURE;
 }
 
 
@@ -824,10 +743,10 @@ doEnvPresets( tOptions* pOpts, teEnvPresetType type )
 STATIC void
 doRcFiles( tOptions* pOpts )
 {
-    int   idx;
-    int   inc = DIRECTION_PRESET;
-    tCC*  pzPath;
-    char  zFileName[ 4096 ];
+    int     idx;
+    int     inc = DIRECTION_PRESET;
+    cch_t*  pzPath;
+    char    zFileName[ 4096 ];
 
     /*
      *  Find the last RC entry (highest priority entry)
@@ -935,6 +854,11 @@ doPresets( tOptions* pOpts )
         (*pOpts->pTransProc)();
     }
 
+    /*
+     *  when comparing long names, these are equivalent
+     */
+    strequate( zSepChars );
+
     {
         const char* pz = strrchr( *pOpts->origArgVect, '/' );
 
@@ -948,6 +872,10 @@ doPresets( tOptions* pOpts )
     if (! SUCCESSFUL( doImmediateOpts( pOpts )))
         return FAILURE;
 
+    /*
+     *  Until we return from this procedure, disable non-presettable opts
+     */
+    pOpts->fOptSet |= OPTPROC_PRESETTING;
     /*
      *  IF there are no RC files,
      *  THEN do any environment presets and leave.
@@ -955,12 +883,14 @@ doPresets( tOptions* pOpts )
     if (  (pOpts->papzHomeList == NULL)
        || SKIP_RC_FILES(pOpts) )  {
         doEnvPresets( pOpts, ENV_ALL );
-        return SUCCESS;
     }
+    else {
+        doEnvPresets( pOpts, ENV_IMM );
+        doRcFiles(    pOpts );
+        doEnvPresets( pOpts, ENV_NON_IMM );
+    }
+    pOpts->fOptSet &= ~OPTPROC_PRESETTING;
 
-    doEnvPresets( pOpts, ENV_IMM );
-    doRcFiles(    pOpts );
-    doEnvPresets( pOpts, ENV_NON_IMM );
     return SUCCESS;
 }
 
@@ -1095,8 +1025,8 @@ checkConsistency( tOptions* pOpts )
  *  These are the routines that libopts users may call directly from their
  *  code.  There are several other routines that can be called by code
  *  generated by the libopts option templates, but they are not to be
- *  called from any other user code.  The @file{options.h} is fairly clear
- *  about this, too.
+ *  called from any other user code.  The @file{options.h} header is
+ *  fairly clear about this, too.
 =*/
 
 /*=export_func optionProcess
@@ -1164,59 +1094,8 @@ optionProcess(
         pOpts->pzCurOpt  = NULL;
     }
 
-    /*
-     *  Now, process all the options from our current position onward.
-     *  (This allows interspersed options and arguments for the few
-     *  non-standard programs that require it.)
-     */
-    for (;;) {
-        tOptState optState = OPTSTATE_INITIALIZER;
-
-        switch (nextOption( pOpts, &optState )) {
-        case FAILURE:
-            if ((pOpts->fOptSet & OPTPROC_ERRSTOP) != 0)
-                (*pOpts->pUsageProc)( pOpts, EXIT_FAILURE );
-            goto optionsBad;
-
-        case PROBLEM:
-            goto optionsDone;
-
-        case SUCCESS:
-            break;
-        }
-
-        /*
-         *  IF this is not an immediate-attribute option, then do it.
-         */
-        switch (optState.flags & (OPTST_DISABLE_IMM|OPTST_IMM)) {
-        case 0:                   /* always */
-            break;
-
-        case OPTST_DISABLE_IMM:   /* disabled options already done */
-            if ((optState.flags & OPTST_DISABLED) != 0)
-                continue;
-            break;
-
-        case OPTST_IMM:           /* enabled options already done */
-            if ((optState.flags & OPTST_DISABLED) == 0)
-                continue;
-            break;
-
-        case OPTST_DISABLE_IMM|OPTST_IMM: /* opt already done */
-            continue;
-        }
-
-        if (! SUCCESSFUL( handleOption( pOpts, &optState ))) {
-            if ((pOpts->fOptSet & OPTPROC_ERRSTOP) != 0)
-                (*pOpts->pUsageProc)( pOpts, EXIT_FAILURE );
-            break;
-        }
-    }
-
- optionsBad:
-    return pOpts->origArgCt;
-
- optionsDone:
+    if (! SUCCESSFUL( doRegularOpts( pOpts )))
+        return pOpts->origArgCt;
 
     /*
      *  IF    there were no errors
