@@ -1,8 +1,8 @@
 
 /*
- *  Time-stamp:      "2004-02-16 11:22:13 bkorb"
+ *  Time-stamp:      "2004-08-30 19:09:50 bkorb"
  *
- *  autoopts.h  $Id: autoopts.h,v 2.46 2004/02/16 22:12:50 bkorb Exp $
+ *  autoopts.h  $Id: autoopts.h,v 2.50 2004/10/02 21:37:46 bkorb Exp $
  *
  *  This file defines all the global structures and special values
  *  used in the automated option processing library.
@@ -68,6 +68,8 @@
 #ifndef MAXPATHLEN
 #  define MAXPATHLEN     4096
 #endif
+#undef  EXPORT
+#define EXPORT
 
 /*
  *  Convert the number to a list usable in a printf call
@@ -177,6 +179,80 @@ typedef struct {
 #  include <snprintfv/printf.h>
 #endif /* AUTOGEN_BUILD */
 
+/*
+ *  DO option handling?
+ *
+ *  Options are examined at two times:  at immediate handling time and at
+ *  normal handling time.  If an option is disabled, the timing may be
+ *  different from the handling of the undisabled option.  The OPTST_DIABLED
+ *  bit indicates the state of the currently discovered option.
+ *  So, here's how it works:
+ *
+ *  A) handling at "immediate" time, either 1 or 2:
+ *
+ *  1.  OPTST_DISABLED is not set:
+ *      IMM           must be set
+ *      DISABLE_IMM   don't care
+ *      TWICE         don't care
+ *      DISABLE_TWICE don't care
+ *      0 -and-  1 x x x
+ *
+ *  2.  OPTST_DISABLED is set:
+ *      IMM           don't care
+ *      DISABLE_IMM   must be set
+ *      TWICE         don't care
+ *      DISABLE_TWICE don't care
+ *      1 -and-  x 1 x x
+ */
+#define DO_IMMEDIATELY(_flg) \
+    (  (((_flg) & (OPTST_DISABLED|OPTST_IMM)) == OPTST_IMM) \
+    || (   ((_flg) & (OPTST_DISABLED|OPTST_DISABLE_IMM))    \
+        == (OPTST_DISABLED|OPTST_DISABLE_IMM)  ))
+
+/*  B) handling at "regular" time because it was not immediate
+ *
+ *  1.  OPTST_DISABLED is not set:
+ *      IMM           must *NOT* be set
+ *      DISABLE_IMM   don't care
+ *      TWICE         don't care
+ *      DISABLE_TWICE don't care
+ *      0 -and-  0 x x x
+ *
+ *  2.  OPTST_DISABLED is set:
+ *      IMM           don't care
+ *      DISABLE_IMM   don't care
+ *      TWICE         must be set
+ *      DISABLE_TWICE don't care
+ *      1 -and-  x x 1 x
+ */
+#define DO_NORMALLY(_flg) ( \
+       (((_flg) & (OPTST_DISABLED|OPTST_IMM))            == 0)  \
+    || (((_flg) & (OPTST_DISABLED|OPTST_DISABLE_IMM))    ==     \
+                  OPTST_DISABLED)  )
+
+/*  C)  handling at "regular" time because it is to be handled twice.
+ *      The immediate bit was already tested and found to be set:
+ *
+ *  3.  OPTST_DISABLED is not set:
+ *      IMM           is set (but don't care)
+ *      DISABLE_IMM   don't care
+ *      TWICE         must be set
+ *      DISABLE_TWICE don't care
+ *      0 -and-  ? x 1 x
+ *
+ *  4.  OPTST_DISABLED is set:
+ *      IMM           don't care
+ *      DISABLE_IMM   is set (but don't care)
+ *      TWICE         don't care
+ *      DISABLE_TWICE must be set
+ *      1 -and-  x ? x 1
+ */
+#define DO_SECOND_TIME(_flg) ( \
+       (((_flg) & (OPTST_DISABLED|OPTST_TWICE))          ==     \
+                  OPTST_TWICE)                                  \
+    || (((_flg) & (OPTST_DISABLED|OPTST_DISABLE_TWICE))  ==     \
+                  (OPTST_DISABLED|OPTST_DISABLE_TWICE)  ))
+
 /*
  *  Define and initialize all the user visible strings.
  *  We do not do translations.  If translations are to be done, then
@@ -206,6 +282,20 @@ longOptionFind( tOptions* pOpts, char* pzOptName, tOptState* pOptState );
 static tSuccess
 shortOptionFind( tOptions* pOpts, tUC optValue, tOptState* pOptState );
 
+static tSuccess
+doImmediateOpts( tOptions* pOpts );
+
+static tSuccess
+doRegularOpts( tOptions* pOpts );
+
+
+/* environment.c */
+static void
+doPrognameEnv( tOptions* pOpts, teEnvPresetType type );
+
+static void
+doEnvPresets( tOptions* pOpts, teEnvPresetType type );
+
 
 /* load.c */
 static void
